<?php
/**
 * A concrete filter for verfying the city name. This filter may intercept the
 * filter chain if no city name is given or if supplied city name has an invalid
 * form. It could also intercept our filter chain if city name was not found.
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2015, 2016 City Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class CityNameVerifierFilter extends BaseFilter implements Filterable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this filter class
	 *
	 * @return	$filterInstance		An instance of this filter class
	 */
	public static final function createCityNameVerifierFilter () {
		// Get a new instance
		$filterInstance = new CityNameVerifierFilter();

		// Return the instance
		return $filterInstance;
	}

	/**
	 * Executes the filter with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 * @throws	FilterChainException	If this filter fails to operate
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get city name from request
		$cityName = $requestInstance->getRequestElement(CityInformationDatabaseWrapper::DB_COLUMN_CITY_NAME);

		// Is the city name set?
		if (is_null($cityName)) {
			// Not found in form so stop the filtering process
			$requestInstance->requestIsValid(FALSE);

			// Add a message to the response
			$responseInstance->addFatalMessage('city_name_unset');

			// Abort here
			throw new FilterChainException($this, self::EXCEPTION_FILTER_CHAIN_INTERCEPTED);
		} elseif (empty($cityName)) {
			// Empty field!
			$requestInstance->requestIsValid(FALSE);

			// Add a message to the response
			$responseInstance->addFatalMessage('city_name_empty');

			// Abort here
			throw new FilterChainException($this, self::EXCEPTION_FILTER_CHAIN_INTERCEPTED);
		} elseif ($this->ifCityNameIsTaken($cityName) === TRUE) {
			// City name is already taken
			$requestInstance->requestIsValid(FALSE);

			// Add a message to the response
			$responseInstance->addFatalMessage('city_name_taken');

			// Abort here
			throw new FilterChainException($this, self::EXCEPTION_FILTER_CHAIN_INTERCEPTED);
		}
	}

	/**
	 * Check whether the city name as already been taken
	 *
	 * @param	$cityName		Cityname to check for existence
	 * @return	$alreadyTaken	Whether the city name has been taken
	 */
	private function ifCityNameIsTaken ($cityName) {
		// Get a new instance
		$managerInstance = ManagerFactory::createManagerByType('city');

		// Make sure the instance is valid
		assert($managerInstance instanceof ManageableCity);

		// Does the city name exist?
		$alreadyTaken = ($managerInstance->ifCityNameExists($cityName));

		// Return the result
		return $alreadyTaken;
	}
}

// [EOF]
?>
